searchNodes=[{"doc":"A bunch of general-purpose helper and convenience functions.","ref":"Bunch.html","title":"Bunch","type":"module"},{"doc":"Imports a bunch of Bunch macros: withl/1 , withl/2 , ~&gt;/2 , ~&gt;&gt;/2 , quote_expr/1 , quote_expr/2","ref":"Bunch.html#__using__/1","title":"Bunch.__using__/1","type":"macro"},{"doc":"Returns an :error tuple if given value is nil and :ok tuple otherwise. Examples iex&gt; map = %{ :answer =&gt; 42 } iex&gt; Bunch . error_if_nil ( map [ :answer ] , :reason ) { :ok , 42 } iex&gt; Bunch . error_if_nil ( map [ :invalid ] , :reason ) { :error , :reason }","ref":"Bunch.html#error_if_nil/2","title":"Bunch.error_if_nil/2","type":"function"},{"doc":"Extracts the key from a key-value tuple.","ref":"Bunch.html#key/1","title":"Bunch.key/1","type":"function"},{"doc":"Embeds the argument in a one-element list if it is not a list itself. Otherwise works as identity. Works similarly to List.wrap/1 , but treats nil as any non-list value, instead of returning empty list in this case. Examples iex&gt; Bunch . listify ( :a ) [ :a ] iex&gt; Bunch . listify ( [ :a , :b , :c ] ) [ :a , :b , :c ] iex&gt; Bunch . listify ( nil ) [ nil ]","ref":"Bunch.html#listify/1","title":"Bunch.listify/1","type":"function"},{"doc":"Creates a short reference.","ref":"Bunch.html#make_short_ref/0","title":"Bunch.make_short_ref/0","type":"function"},{"doc":"Works like quote/2 , but doesn't require a do/end block and options are passed as the last argument. Useful when quoting a single expression. Examples iex&gt; use Bunch iex&gt; quote_expr ( String . t ( ) ) quote do String . t ( ) end iex&gt; quote_expr ( unquote ( x ) + 2 , unquote : false ) quote unquote : false do unquote ( x ) + 2 end Nesting Nesting calls to quote disables unquoting in the inner call, while placing quote_expr in quote or another quote_expr does not: iex&gt; use Bunch iex&gt; quote do quote do unquote ( :code ) end end == quote do quote do :code end end false iex&gt; quote do quote_expr ( unquote ( :code ) ) end == quote do quote_expr ( :code ) end true","ref":"Bunch.html#quote_expr/2","title":"Bunch.quote_expr/2","type":"macro"},{"doc":"Returns given stateful try value along with its status.","ref":"Bunch.html#stateful_try_with_status/1","title":"Bunch.stateful_try_with_status/1","type":"function"},{"doc":"Extracts the value from a key-value tuple.","ref":"Bunch.html#value/1","title":"Bunch.value/1","type":"function"},{"doc":"Works like withl/2 , but allows shorter syntax. Examples iex&gt; use Bunch iex&gt; x = 1 iex&gt; y = 2 iex&gt; withl a : true &lt;- x &gt; 0 , ...&gt; b : false &lt;- y |&gt; rem ( 2 ) == 0 , ...&gt; do : { x , y } , ...&gt; else : ( a : false -&gt; { :error , :x } ; b : true -&gt; { :error , :y } ) { :error , :y } For more details and more verbose and readable syntax, check docs for withl/2 .","ref":"Bunch.html#withl/1","title":"Bunch.withl/1","type":"macro"},{"doc":"A labeled version of the with/1 macro. This macro works like with/1 , but enforces user to mark corresponding withl and else clauses with the same label (atom). If a withl clause does not match, only the else clauses marked with the same label are matched against the result. iex&gt; use Bunch iex&gt; names = %{ 1 =&gt; &quot;Harold&quot; , 2 =&gt; &quot;Małgorzata&quot; } iex&gt; test = fn id -&gt; ...&gt; withl id : { int_id , _ } &lt;- Integer . parse ( id ) , ...&gt; name : { :ok , name } &lt;- Map . fetch ( names , int_id ) do ...&gt; { :ok , &quot;The name is \#{ name } &quot; } ...&gt; else ...&gt; id : :error -&gt; { :error , :invalid_id } ...&gt; name : :error -&gt; { :error , :name_not_found } ...&gt; end ...&gt; end iex&gt; test . ( &quot;1&quot; ) { :ok , &quot;The name is Harold&quot; } iex&gt; test . ( &quot;5&quot; ) { :error , :name_not_found } iex&gt; test . ( &quot;something&quot; ) { :error , :invalid_id } withl clauses using no &lt;- operator are supported, but they also have to be labeled due to Elixir syntax restrictions. iex&gt; use Bunch iex&gt; names = %{ 1 =&gt; &quot;Harold&quot; , 2 =&gt; &quot;Małgorzata&quot; } iex&gt; test = fn id -&gt; ...&gt; withl id : { int_id , _ } &lt;- Integer . parse ( id ) , ...&gt; do : int_id = int_id + 1 , ...&gt; name : { :ok , name } &lt;- Map . fetch ( names , int_id ) do ...&gt; { :ok , &quot;The name is \#{ name } &quot; } ...&gt; else ...&gt; id : :error -&gt; { :error , :invalid_id } ...&gt; name : :error -&gt; { :error , :name_not_found } ...&gt; end ...&gt; end iex&gt; test . ( &quot;0&quot; ) { :ok , &quot;The name is Harold&quot; } All the withl clauses that use &lt;- operator must have at least one corresponding else clause. iex&gt; use Bunch iex&gt; try do ...&gt; Code . compile_quoted ( quote do ...&gt; withl a : a when a &gt; 0 &lt;- 1 , ...&gt; b : b when b &gt; 0 &lt;- 2 do ...&gt; { :ok , a + b } ...&gt; else ...&gt; a : _ -&gt; :error ...&gt; end ...&gt; end ) ...&gt; rescue ...&gt; e -&gt; e . description ...&gt; end &quot;Label `:b` not present in withl else clauses&quot; Variable scoping Because the labels are resolved in the compile time, they make it possible to access results of already succeeded matches from else clauses. This may help handling errors, like below: iex&gt; use Bunch iex&gt; names = %{ 1 =&gt; &quot;Harold&quot; , 2 =&gt; &quot;Małgorzata&quot; } iex&gt; test = fn id -&gt; ...&gt; withl id : { int_id , _ } &lt;- Integer . parse ( id ) , ...&gt; do : int_id = int_id + 1 , ...&gt; name : { :ok , name } &lt;- Map . fetch ( names , int_id ) do ...&gt; { :ok , &quot;The name is \#{ name } &quot; } ...&gt; else ...&gt; id : :error -&gt; { :error , :invalid_id } ...&gt; name : :error -&gt; { :ok , &quot;The name is Defaultius the \#{ int_id } th&quot; } ...&gt; end ...&gt; end iex&gt; test . ( &quot;0&quot; ) { :ok , &quot;The name is Harold&quot; } iex&gt; test . ( &quot;5&quot; ) { :ok , &quot;The name is Defaultius the 6th&quot; } Duplicate labels withl supports marking multiple withl clauses with the same label, however in that case all the else clauses marked with such label are simply put multiple times into the generated code. Note that this may lead to confusion, in particular when variables are rebound in withl clauses: iex&gt; use Bunch iex&gt; test = fn x -&gt; ...&gt; withl a : x when x &gt; 1 &lt;- x , ...&gt; do : x = x + 1 , ...&gt; a : x when x &lt; 4 &lt;- x do ...&gt; :ok ...&gt; else ...&gt; a : x -&gt; { :error , x } ...&gt; end ...&gt; end iex&gt; test . ( 2 ) :ok iex&gt; test . ( 1 ) { :error , 1 } iex&gt; test . ( 3 ) { :error , 4 }","ref":"Bunch.html#withl/2","title":"Bunch.withl/2","type":"macro"},{"doc":"Helper for writing pipeline-like syntax. Maps given value using match clauses or lambda-like syntax. Examples iex&gt; use Bunch iex&gt; { :ok , 10 } ~&gt; ( { :ok , x } -&gt; x ) 10 iex&gt; 5 ~&gt; &amp;1 + 2 7 Lambda-like expressions are not converted to lambdas under the hood, but result of expr is injected to &amp;1 at the compile time. Useful especially when dealing with a pipeline of operations (made up e.g. with pipe ( |&gt; ) operator) some of which are hard to express in such form: iex&gt; use Bunch iex&gt; [ &quot;Joe&quot; , &quot;truck&quot; , &quot;jacket&quot; ] ...&gt; |&gt; Enum . map ( &amp; String . downcase / 1 ) ...&gt; |&gt; Enum . filter ( &amp; &amp;1 |&gt; String . starts_with? ( &quot;j&quot; ) ) ...&gt; ~&gt; [ &quot;Words:&quot; | &amp;1 ] ...&gt; |&gt; Enum . join ( &quot; \\n &quot; ) &quot;Words: joe jacket&quot;","ref":"Bunch.html#~%3E/2","title":"Bunch.~>/2","type":"macro"},{"doc":"Works similar to ~&gt;/2 , but accepts only -&gt; clauses and appends default identity clause at the end. Examples iex&gt; use Bunch iex&gt; { :ok , 10 } ~&gt;&gt; ( { :ok , x } -&gt; { :ok , x + 1 } ) { :ok , 11 } iex&gt; :error ~&gt;&gt; ( { :ok , x } -&gt; { :ok , x + 1 } ) :error","ref":"Bunch.html#~%3E%3E/2","title":"Bunch.~>>/2","type":"macro"},{"doc":"A bunch of functions for easier manipulation on terms of types implementing Access behaviour.","ref":"Bunch.Access.html","title":"Bunch.Access","type":"module"},{"doc":"Implements Access behaviour by delegating callbacks to Map module. All the callbacks are overridable.","ref":"Bunch.Access.html#__using__/1","title":"Bunch.Access.__using__/1","type":"macro"},{"doc":"Works like pop_in/2 , but discards returned value. Examples iex&gt; Bunch.Access . delete_in ( %{ a : %{ b : 10 } } , [ :a , :b ] ) %{ a : %{ } } iex&gt; Bunch.Access . delete_in ( %{ a : 10 } , :a ) %{ } iex&gt; Bunch.Access . delete_in ( 10 , [ ] ) nil","ref":"Bunch.Access.html#delete_in/2","title":"Bunch.Access.delete_in/2","type":"function"},{"doc":"Works like Kernel.get_and_update_in/3 with small differences. Behaviour differs in the following aspects: empty lists of keys are allowed single key does not have to be wrapped in a list Examples iex&gt; Bunch.Access . get_and_update_in ( %{ a : %{ b : 10 } } , [ :a , :b ] , &amp; { &amp;1 , &amp;1 * 2 } ) { 10 , %{ a : %{ b : 20 } } } iex&gt; Bunch.Access . get_and_update_in ( %{ a : 10 } , :a , &amp; { &amp;1 , &amp;1 * 2 } ) { 10 , %{ a : 20 } } iex&gt; Bunch.Access . get_and_update_in ( 10 , [ ] , &amp; { &amp;1 , &amp;1 * 2 } ) { 10 , 20 }","ref":"Bunch.Access.html#get_and_update_in/3","title":"Bunch.Access.get_and_update_in/3","type":"function"},{"doc":"Works like Kernel.get_in/2 with small differences. Behaviour differs in the following aspects: empty lists of keys are allowed single key does not have to be wrapped in a list Examples iex&gt; Bunch.Access . get_in ( %{ a : %{ b : 10 } } , [ :a , :b ] ) 10 iex&gt; Bunch.Access . get_in ( %{ a : 10 } , :a ) 10 iex&gt; Bunch.Access . get_in ( %{ a : %{ b : 10 } } , [ ] ) %{ a : %{ b : 10 } }","ref":"Bunch.Access.html#get_in/2","title":"Bunch.Access.get_in/2","type":"function"},{"doc":"Updates value at keys in a nested data structure and returns new value and updated structure. Uses get_and_update_in/3 under the hood. Example iex&gt; %{ a : %{ b : 10 } } |&gt; Bunch.Access . get_updated_in ( [ :a , :b ] , &amp; &amp;1 + 1 ) { 11 , %{ a : %{ b : 11 } } }","ref":"Bunch.Access.html#get_updated_in/3","title":"Bunch.Access.get_updated_in/3","type":"function"},{"doc":"Works like Kernel.pop_in/2 with small differences. Behaviour differs in the following aspects: empty lists of keys are allowed single key does not have to be wrapped in a list Examples iex&gt; Bunch.Access . pop_in ( %{ a : %{ b : 10 } } , [ :a , :b ] ) { 10 , %{ a : %{ } } } iex&gt; Bunch.Access . pop_in ( %{ a : 10 } , :a ) { 10 , %{ } } iex&gt; Bunch.Access . pop_in ( 10 , [ ] ) { 10 , nil }","ref":"Bunch.Access.html#pop_in/2","title":"Bunch.Access.pop_in/2","type":"function"},{"doc":"Works like Kernel.put_in/3 with small differences. Behaviour differs in the following aspects: empty lists of keys are allowed single key does not have to be wrapped in a list Examples iex&gt; Bunch.Access . put_in ( %{ a : %{ b : 10 } } , [ :a , :b ] , 20 ) %{ a : %{ b : 20 } } iex&gt; Bunch.Access . put_in ( %{ a : 10 } , :a , 20 ) %{ a : 20 } iex&gt; Bunch.Access . put_in ( %{ a : %{ b : 10 } } , [ ] , 20 ) 20","ref":"Bunch.Access.html#put_in/3","title":"Bunch.Access.put_in/3","type":"function"},{"doc":"Works like Kernel.update_in/3 with small differences. Behaviour differs in the following aspects: empty lists of keys are allowed single key does not have to be wrapped in a list Examples iex&gt; Bunch.Access . update_in ( %{ a : %{ b : 10 } } , [ :a , :b ] , &amp; &amp;1 * 2 ) %{ a : %{ b : 20 } } iex&gt; Bunch.Access . update_in ( %{ a : 10 } , :a , &amp; &amp;1 * 2 ) %{ a : 20 } iex&gt; Bunch.Access . update_in ( 10 , [ ] , &amp; &amp;1 * 2 ) 20","ref":"Bunch.Access.html#update_in/3","title":"Bunch.Access.update_in/3","type":"function"},{"doc":"A bunch of helpers for manipulating binaries.","ref":"Bunch.Binary.html","title":"Bunch.Binary","type":"module"},{"doc":"Chunks given binary into parts of given size. Remaining part is cut off. Examples iex&gt; &lt;&lt; 1 , 2 , 3 , 4 , 5 , 6 &gt;&gt; |&gt; Bunch.Binary . chunk_every ( 2 ) [ &lt;&lt; 1 , 2 &gt;&gt; , &lt;&lt; 3 , 4 &gt;&gt; , &lt;&lt; 5 , 6 &gt;&gt; ] iex&gt; &lt;&lt; 1 , 2 , 3 , 4 , 5 , 6 , 7 &gt;&gt; |&gt; Bunch.Binary . chunk_every ( 2 ) [ &lt;&lt; 1 , 2 &gt;&gt; , &lt;&lt; 3 , 4 &gt;&gt; , &lt;&lt; 5 , 6 &gt;&gt; ]","ref":"Bunch.Binary.html#chunk_every/2","title":"Bunch.Binary.chunk_every/2","type":"function"},{"doc":"Chunks given binary into parts of given size. Returns list of chunks and remainder. Examples iex&gt; &lt;&lt; 1 , 2 , 3 , 4 , 5 , 6 &gt;&gt; |&gt; Bunch.Binary . chunk_every_rem ( 2 ) { [ &lt;&lt; 1 , 2 &gt;&gt; , &lt;&lt; 3 , 4 &gt;&gt; , &lt;&lt; 5 , 6 &gt;&gt; ] , &lt;&lt; &gt;&gt; } iex&gt; &lt;&lt; 1 , 2 , 3 , 4 , 5 , 6 , 7 &gt;&gt; |&gt; Bunch.Binary . chunk_every_rem ( 2 ) { [ &lt;&lt; 1 , 2 &gt;&gt; , &lt;&lt; 3 , 4 &gt;&gt; , &lt;&lt; 5 , 6 &gt;&gt; ] , &lt;&lt; 7 &gt;&gt; }","ref":"Bunch.Binary.html#chunk_every_rem/2","title":"Bunch.Binary.chunk_every_rem/2","type":"function"},{"doc":"Returns a 2-tuple, where the first element is the result of take_int_part(binary, i) , and the second is the rest of binary . Examples iex&gt; import Bunch.Binary iex&gt; split_int_part ( &lt;&lt; 1 , 2 , 3 , 4 , 5 , 6 , 7 , 8 &gt;&gt; , 3 ) { &lt;&lt; 1 , 2 , 3 , 4 , 5 , 6 &gt;&gt; , &lt;&lt; 7 , 8 &gt;&gt; } iex&gt; split_int_part ( &lt;&lt; 1 , 2 , 3 , 4 , 5 , 6 , 7 , 8 &gt;&gt; , 4 ) { &lt;&lt; 1 , 2 , 3 , 4 , 5 , 6 , 7 , 8 &gt;&gt; , &lt;&lt; &gt;&gt; }","ref":"Bunch.Binary.html#split_int_part/2","title":"Bunch.Binary.split_int_part/2","type":"function"},{"doc":"Cuts off the smallest possible chunk from the end of binary , so that the size of returned binary is an integer multiple of i . Examples iex&gt; import Bunch.Binary iex&gt; take_int_part ( &lt;&lt; 1 , 2 , 3 , 4 , 5 , 6 , 7 , 8 &gt;&gt; , 3 ) &lt;&lt; 1 , 2 , 3 , 4 , 5 , 6 &gt;&gt; iex&gt; take_int_part ( &lt;&lt; 1 , 2 , 3 , 4 , 5 , 6 , 7 , 8 &gt;&gt; , 4 ) &lt;&lt; 1 , 2 , 3 , 4 , 5 , 6 , 7 , 8 &gt;&gt;","ref":"Bunch.Binary.html#take_int_part/2","title":"Bunch.Binary.take_int_part/2","type":"function"},{"doc":"A bunch of helper functions for code compilation, code evaluation, and code loading.","ref":"Bunch.Code.html","title":"Bunch.Code","type":"module"},{"doc":"Takes a code block, expands macros inside and pretty prints it.","ref":"Bunch.Code.html#peek_code/1","title":"Bunch.Code.peek_code/1","type":"macro"},{"doc":"Returns stacktrace as a string. The stacktrace is formatted to the readable format.","ref":"Bunch.Code.html#stacktrace/0","title":"Bunch.Code.stacktrace/0","type":"macro"},{"doc":"A bunch of helpers for parsing and validating configurations.","ref":"Bunch.Config.html","title":"Bunch.Config","type":"module"},{"doc":"Parses config according to fields_specs . fields_specs consist of constraints on each field. Supported constraints are: validate - function determining if field's value is correct in - enumerable containing all valid values default - value returned if a field is not found in config require_if - function determining if a field is required basing on previous fields' values Examples iex&gt; Bunch.Config . parse ( [ a : 1 , b : 2 ] , a : [ validate : &amp; &amp;1 &gt; 0 ] , b : [ in : - 2 .. 2 ] ) { :ok , %{ a : 1 , b : 2 } } iex&gt; Bunch.Config . parse ( [ a : 1 , b : 4 ] , a : [ validate : &amp; &amp;1 &gt; 0 ] , b : [ in : - 2 .. 2 ] ) { :error , { :config_field , { :invalid_value , [ key : :b , value : 4 , reason : { :not_in , - 2 .. 2 } ] } } } iex&gt; Bunch.Config . parse ( ...&gt; [ a : 1 , b : 2 ] , ...&gt; a : [ validate : &amp; &amp;1 &gt; 0 ] , ...&gt; b : [ in : - 2 .. 2 ] , ...&gt; c : [ default : 5 ] ...&gt; ) { :ok , %{ a : 1 , b : 2 , c : 5 } } iex&gt; Bunch.Config . parse ( ...&gt; [ a : 1 , b : 2 ] , ...&gt; a : [ validate : &amp; &amp;1 &gt; 0 ] , ...&gt; b : [ in : - 2 .. 2 ] , ...&gt; c : [ require_if : &amp; &amp;1 . a == &amp;1 . b ] ...&gt; ) { :ok , %{ a : 1 , b : 2 } } iex&gt; Bunch.Config . parse ( ...&gt; [ a : 1 , b : 1 ] , ...&gt; a : [ validate : &amp; &amp;1 &gt; 0 ] , ...&gt; b : [ in : - 2 .. 2 ] , ...&gt; c : [ require_if : &amp; &amp;1 . a == &amp;1 . b ] ...&gt; ) { :error , { :config_field , { :key_not_found , :c } } }","ref":"Bunch.Config.html#parse/2","title":"Bunch.Config.parse/2","type":"function"},{"doc":"A bunch of helper functions for manipulating enums.","ref":"Bunch.Enum.html","title":"Bunch.Enum","type":"module"},{"doc":"Splits enumerable into chunks, and passes each chunk through collector . New chunk is created each time chunker returns false . The chunker is passed current and previous element of enumerable. Examples: iex&gt; Bunch.Enum . chunk_by_prev ( [ 1 , 2 , 5 , 5 ] , fn x , y -&gt; x - y &lt;= 2 end ) [ [ 1 , 2 ] , [ 5 , 5 ] ] iex&gt; Bunch.Enum . chunk_by_prev ( [ 1 , 2 , 5 , 5 ] , fn x , y -&gt; x - y &lt;= 2 end , &amp; Enum . sum / 1 ) [ 3 , 10 ]","ref":"Bunch.Enum.html#chunk_by_prev/3","title":"Bunch.Enum.chunk_by_prev/3","type":"function"},{"doc":"Returns elements that occur at least min_occurences times in enumerable. Results are NOT ordered in any sensible way, neither is the order anyhow preserved, but it is deterministic. Examples iex&gt; Bunch.Enum . duplicates ( [ 1 , 3 , 2 , 5 , 3 , 2 , 2 ] ) [ 2 , 3 ] iex&gt; Bunch.Enum . duplicates ( [ 1 , 3 , 2 , 5 , 3 , 2 , 2 ] , 3 ) [ 2 ]","ref":"Bunch.Enum.html#duplicates/2","title":"Bunch.Enum.duplicates/2","type":"function"},{"doc":"Generates a list by calling i times function f . iex&gt; { :ok , pid } = Agent . start_link ( fn -&gt; 0 end ) iex&gt; Bunch.Enum . repeat ( fn -&gt; Agent . get_and_update ( pid , &amp; { &amp;1 , &amp;1 + 1 } ) end , 4 ) [ 0 , 1 , 2 , 3 ] iex&gt; Bunch.Enum . repeat ( fn -&gt; :abc end , 0 ) [ ]","ref":"Bunch.Enum.html#repeat/2","title":"Bunch.Enum.repeat/2","type":"function"},{"doc":"Generates a list consisting of i values v . iex&gt; Bunch.Enum . repeated ( :abc , 4 ) [ :abc , :abc , :abc , :abc ] iex&gt; Bunch.Enum . repeated ( :abc , 0 ) [ ]","ref":"Bunch.Enum.html#repeated/2","title":"Bunch.Enum.repeated/2","type":"function"},{"doc":"Works like Enum.each/2 , but breaks on error. Behaves like Enum.each/2 as long as given fun returns :ok . If it happens to return {:error, reason} , traversal is stopped and the error is returned. Examples: iex&gt; fun = fn 0 -&gt; { :error , :zero } ; x -&gt; send ( self ( ) , 1 / x ) ; :ok end iex&gt; Bunch.Enum . try_each ( [ 1 , 2 , 3 ] , fun ) :ok iex&gt; Bunch.Enum . try_each ( [ 1 , 0 , 3 ] , fun ) { :error , :zero }","ref":"Bunch.Enum.html#try_each/2","title":"Bunch.Enum.try_each/2","type":"function"},{"doc":"Works like Enum.flat_map/2 , but breaks on error. Behaves like Enum.flat_map/2 as long as reducing function returns {:ok, values} . If it happens to return {:error, reason} , reduction is stopped and the error is returned. Examples: iex&gt; fun = fn 0 -&gt; { :error , :zero } ; x -&gt; { :ok , [ 1 / x , 2 / x , 3 / x ] } end iex&gt; Bunch.Enum . try_flat_map ( [ 1 , 5 , - 2 , 8 ] , fun ) { :ok , [ 1.0 , 2.0 , 3.0 , 0.2 , 0.4 , 0.6 , - 0.5 , - 1.0 , - 1.5 , 0.125 , 0.25 , 0.375 ] } iex&gt; Bunch.Enum . try_flat_map ( [ 1 , 5 , 0 , 8 ] , fun ) { :error , :zero }","ref":"Bunch.Enum.html#try_flat_map/2","title":"Bunch.Enum.try_flat_map/2","type":"function"},{"doc":"Works like Enum.each/2 , but breaks on error. Behaves like Enum.flat_map_reduce/3 as long as given fun returns {{:ok, value}, new_acc} . If it happens to return {{:error, reason}, new_acc} , reduction is stopped and the error is returned. Examples: iex&gt; fun = fn ...&gt; x , acc when acc &gt;= 0 -&gt; { { :ok , [ x + 1 , x + 2 , x + 3 ] } , x + acc } ...&gt; _ , acc -&gt; { { :error , :negative_prefix_sum } , acc } ...&gt; end iex&gt; Bunch.Enum . try_flat_map_reduce ( [ 1 , 5 , - 2 , 8 ] , 0 , fun ) { { :ok , [ 2 , 3 , 4 , 6 , 7 , 8 , - 1 , 0 , 1 , 9 , 10 , 11 ] } , 12 } iex&gt; Bunch.Enum . try_flat_map_reduce ( [ 1 , 5 , - 7 , 8 ] , 0 , fun ) { { :error , :negative_prefix_sum } , - 1 }","ref":"Bunch.Enum.html#try_flat_map_reduce/3","title":"Bunch.Enum.try_flat_map_reduce/3","type":"function"},{"doc":"Works like Enum.map/2 , but breaks on error. Behaves like Enum.map/2 as long as given fun returns {:ok, value} . If it happens to return {:error, reason} , reduction is stopped and the error is returned. Examples: iex&gt; fun = fn 0 -&gt; { :error , :zero } ; x -&gt; { :ok , 1 / x } end iex&gt; Bunch.Enum . try_map ( [ 1 , 5 , - 2 , 8 ] , fun ) { :ok , [ 1.0 , 0.2 , - 0.5 , 0.125 ] } iex&gt; Bunch.Enum . try_map ( [ 1 , 5 , 0 , 8 ] , fun ) { :error , :zero }","ref":"Bunch.Enum.html#try_map/2","title":"Bunch.Enum.try_map/2","type":"function"},{"doc":"Works like Enum.map_reduce/3 , but breaks on error. Behaves like Enum.map_reduce/3 as long as given fun returns {{:ok, value}, new_acc} . If it happens to return {{:error, reason}, new_acc} , reduction is stopped and the error is returned. Examples: iex&gt; fun = fn ...&gt; x , acc when acc &gt;= 0 -&gt; { { :ok , x + 1 } , x + acc } ...&gt; _ , acc -&gt; { { :error , :negative_prefix_sum } , acc } ...&gt; end iex&gt; Bunch.Enum . try_map_reduce ( [ 1 , 5 , - 2 , 8 ] , 0 , fun ) { { :ok , [ 2 , 6 , - 1 , 9 ] } , 12 } iex&gt; Bunch.Enum . try_map_reduce ( [ 1 , 5 , - 7 , 8 ] , 0 , fun ) { { :error , :negative_prefix_sum } , - 1 }","ref":"Bunch.Enum.html#try_map_reduce/3","title":"Bunch.Enum.try_map_reduce/3","type":"function"},{"doc":"Works like Enum.reduce/3 , but breaks on error. Behaves like Enum.reduce/3 as long as given fun returns {:ok, new_acc} . If it happens to return {{:error, reason}, new_acc} , reduction is stopped and the error is returned. Examples: iex&gt; fun = fn ...&gt; x , acc when acc &gt;= 0 -&gt; { :ok , x + acc } ...&gt; _ , acc -&gt; { { :error , :negative_prefix_sum } , acc } ...&gt; end iex&gt; Bunch.Enum . try_reduce ( [ 1 , 5 , - 2 , 8 ] , 0 , fun ) { :ok , 12 } iex&gt; Bunch.Enum . try_reduce ( [ 1 , 5 , - 7 , 8 ] , 0 , fun ) { { :error , :negative_prefix_sum } , - 1 }","ref":"Bunch.Enum.html#try_reduce/3","title":"Bunch.Enum.try_reduce/3","type":"function"},{"doc":"Works like Enum.reduce_while/3 , but breaks on error. Behaves like Enum.reduce_while/3 as long as given fun returns {{:ok, :cont | :halt}, new_acc} . If it happens to return {{:error, reason}, new_acc} , reduction is stopped and the error is returned. Examples: iex&gt; fun = fn ...&gt; 0 , acc -&gt; { { :ok , :halt } , acc } ...&gt; x , acc when acc &gt;= 0 -&gt; { { :ok , :cont } , x + acc } ...&gt; _ , acc -&gt; { { :error , :negative_prefix_sum } , acc } ...&gt; end iex&gt; Bunch.Enum . try_reduce_while ( [ 1 , 5 , - 2 , 8 ] , 0 , fun ) { :ok , 12 } iex&gt; Bunch.Enum . try_reduce_while ( [ 1 , 5 , 0 , 8 ] , 0 , fun ) { :ok , 6 } iex&gt; Bunch.Enum . try_reduce_while ( [ 1 , 5 , - 7 , 8 ] , 0 , fun ) { { :error , :negative_prefix_sum } , - 1 }","ref":"Bunch.Enum.html#try_reduce_while/3","title":"Bunch.Enum.try_reduce_while/3","type":"function"},{"doc":"Implementation of Enum.unzip/1 for more-than-two-element tuples. Size of returned tuple is equal to size of the shortest tuple in tuples . Examples: iex&gt; Bunch.Enum . unzip ( [ { 1 , 2 , 3 } , { 4 , 5 , 6 } , { 7 , 8 , 9 } , { 10 , 11 , 12 } ] ) { [ 1 , 4 , 7 , 10 ] , [ 2 , 5 , 8 , 11 ] , [ 3 , 6 , 9 , 12 ] } iex&gt; Bunch.Enum . unzip ( [ { 1 , 2 , 3 } , { 4 , 5 } , { 6 , 7 , 8 , 9 } , { 10 , 11 , 12 } ] ) { [ 1 , 4 , 6 , 10 ] , [ 2 , 5 , 7 , 11 ] }","ref":"Bunch.Enum.html#unzip/1","title":"Bunch.Enum.unzip/1","type":"function"},{"doc":"Works the same way as Enum.zip/1 , but does not cut off remaining values. Examples: iex&gt; Bunch.Enum . zip_longest ( [ [ 1 , 2 ] , [ 3 , 4 , 5 ] ] ) [ [ 1 , 3 ] , [ 2 , 4 ] , [ 5 ] ] It also returns list of lists, as opposed to tuples.","ref":"Bunch.Enum.html#zip_longest/1","title":"Bunch.Enum.zip_longest/1","type":"function"},{"doc":"A bunch of helper functions for manipulating key-value enums (including keyword enums). Key-value enums are represented as enums of 2-element tuples, where the first element of each tuple is a key, and the second is a value.","ref":"Bunch.KVEnum.html","title":"Bunch.KVEnum","type":"module"},{"doc":"Returns true if f returns truthy value for any key from enum , otherwise false . Example iex&gt; Bunch.KVEnum . any_key? ( [ a : 1 , b : 2 , a : 3 ] , &amp; &amp;1 == :b ) true iex&gt; Bunch.KVEnum . any_key? ( [ a : 1 , b : 3 , a : 5 ] , &amp; &amp;1 == :c ) false","ref":"Bunch.KVEnum.html#any_key?/2","title":"Bunch.KVEnum.any_key?/2","type":"function"},{"doc":"Returns true if f returns truthy value for any value from enum , otherwise false . Example iex&gt; Bunch.KVEnum . any_value? ( [ a : 1 , b : 2 , a : 3 ] , &amp; &amp;1 |&gt; rem ( 2 ) == 0 ) true iex&gt; Bunch.KVEnum . any_value? ( [ a : 1 , b : 3 , a : 5 ] , &amp; &amp;1 |&gt; rem ( 2 ) == 0 ) false","ref":"Bunch.KVEnum.html#any_value?/2","title":"Bunch.KVEnum.any_value?/2","type":"function"},{"doc":"Executes f for each key in enum . Example iex&gt; Bunch.KVEnum . each_key ( [ a : 1 , b : 2 , a : 3 ] , &amp; send ( self ( ) , &amp;1 ) ) iex&gt; [ :a , :b , :a ] |&gt; Enum . each ( &amp; receive do ^ &amp;1 -&gt; :ok end ) :ok","ref":"Bunch.KVEnum.html#each_key/2","title":"Bunch.KVEnum.each_key/2","type":"function"},{"doc":"Executes f for each value in enum . Example iex&gt; Bunch.KVEnum . each_value ( [ a : 1 , b : 2 , a : 3 ] , &amp; send ( self ( ) , &amp;1 ) ) iex&gt; 1 .. 3 |&gt; Enum . each ( &amp; receive do ^ &amp;1 -&gt; :ok end ) :ok","ref":"Bunch.KVEnum.html#each_value/2","title":"Bunch.KVEnum.each_value/2","type":"function"},{"doc":"Filters elements of enum by keys using function f . Example iex&gt; Bunch.KVEnum . filter_by_keys ( [ a : 1 , b : 2 , a : 3 ] , &amp; &amp;1 == :a ) [ a : 1 , a : 3 ]","ref":"Bunch.KVEnum.html#filter_by_keys/2","title":"Bunch.KVEnum.filter_by_keys/2","type":"function"},{"doc":"Filters elements of enum by values using function f . Example iex&gt; Bunch.KVEnum . filter_by_values ( [ a : 1 , b : 2 , a : 3 ] , &amp; &amp;1 |&gt; rem ( 2 ) == 0 ) [ b : 2 ]","ref":"Bunch.KVEnum.html#filter_by_values/2","title":"Bunch.KVEnum.filter_by_values/2","type":"function"},{"doc":"Returns all keys from the enum . Duplicated keys appear duplicated in the final enum of keys. Examples iex&gt; Bunch.KVEnum . keys ( a : 1 , b : 2 ) [ :a , :b ] iex&gt; Bunch.KVEnum . keys ( a : 1 , b : 2 , a : 3 ) [ :a , :b , :a ]","ref":"Bunch.KVEnum.html#keys/1","title":"Bunch.KVEnum.keys/1","type":"function"},{"doc":"Maps keys of enum using function f . Example iex&gt; Bunch.KVEnum . map_keys ( [ { 1 , :a } , { 2 , :b } ] , &amp; &amp;1 + 1 ) [ { 2 , :a } , { 3 , :b } ]","ref":"Bunch.KVEnum.html#map_keys/2","title":"Bunch.KVEnum.map_keys/2","type":"function"},{"doc":"Maps values of enum using function f . Example iex&gt; Bunch.KVEnum . map_values ( [ a : 1 , b : 2 ] , &amp; &amp;1 + 1 ) [ a : 2 , b : 3 ]","ref":"Bunch.KVEnum.html#map_values/2","title":"Bunch.KVEnum.map_values/2","type":"function"},{"doc":"Returns all values from the enum . Values from duplicated keys will be kept in the final enum of values. Examples iex&gt; Bunch.KVEnum . values ( a : 1 , b : 2 ) [ 1 , 2 ] iex&gt; Bunch.KVEnum . values ( a : 1 , b : 2 , a : 3 ) [ 1 , 2 , 3 ]","ref":"Bunch.KVEnum.html#values/1","title":"Bunch.KVEnum.values/1","type":"function"},{"doc":"","ref":"Bunch.KVEnum.html#t:t/2","title":"Bunch.KVEnum.t/2","type":"type"},{"doc":"A bunch of helper functions for manipulating key-value lists (including keyword lists). Key-value lists are represented as lists of 2-element tuples, where the first element of each tuple is a key, and the second is a value.","ref":"Bunch.KVList.html","title":"Bunch.KVList","type":"module"},{"doc":"Returns true if f returns truthy value for any key from list , otherwise false . Example iex&gt; Bunch.KVList . any_key? ( [ a : 1 , b : 2 , a : 3 ] , &amp; &amp;1 == :b ) true iex&gt; Bunch.KVList . any_key? ( [ a : 1 , b : 3 , a : 5 ] , &amp; &amp;1 == :c ) false","ref":"Bunch.KVList.html#any_key?/2","title":"Bunch.KVList.any_key?/2","type":"function"},{"doc":"Returns true if f returns truthy value for any value from list , otherwise false . Example iex&gt; Bunch.KVList . any_value? ( [ a : 1 , b : 2 , a : 3 ] , &amp; &amp;1 |&gt; rem ( 2 ) == 0 ) true iex&gt; Bunch.KVList . any_value? ( [ a : 1 , b : 3 , a : 5 ] , &amp; &amp;1 |&gt; rem ( 2 ) == 0 ) false","ref":"Bunch.KVList.html#any_value?/2","title":"Bunch.KVList.any_value?/2","type":"function"},{"doc":"Executes f for each key in list . Example iex&gt; Bunch.KVList . each_key ( [ a : 1 , b : 2 , a : 3 ] , &amp; send ( self ( ) , &amp;1 ) ) iex&gt; [ :a , :b , :a ] |&gt; Enum . each ( &amp; receive do ^ &amp;1 -&gt; :ok end ) :ok","ref":"Bunch.KVList.html#each_key/2","title":"Bunch.KVList.each_key/2","type":"function"},{"doc":"Executes f for each value in list . Example iex&gt; Bunch.KVList . each_value ( [ a : 1 , b : 2 , a : 3 ] , &amp; send ( self ( ) , &amp;1 ) ) iex&gt; 1 .. 3 |&gt; Enum . each ( &amp; receive do ^ &amp;1 -&gt; :ok end ) :ok","ref":"Bunch.KVList.html#each_value/2","title":"Bunch.KVList.each_value/2","type":"function"},{"doc":"Filters elements of list by keys using function f . Example iex&gt; Bunch.KVList . filter_by_keys ( [ a : 1 , b : 2 , a : 3 ] , &amp; &amp;1 == :a ) [ a : 1 , a : 3 ]","ref":"Bunch.KVList.html#filter_by_keys/2","title":"Bunch.KVList.filter_by_keys/2","type":"function"},{"doc":"Filters elements of list by values using function f . Example iex&gt; Bunch.KVList . filter_by_values ( [ a : 1 , b : 2 , a : 3 ] , &amp; &amp;1 |&gt; rem ( 2 ) == 0 ) [ b : 2 ]","ref":"Bunch.KVList.html#filter_by_values/2","title":"Bunch.KVList.filter_by_values/2","type":"function"},{"doc":"Maps keys of list using function f . Example iex&gt; Bunch.KVList . map_keys ( [ { 1 , :a } , { 2 , :b } ] , &amp; &amp;1 + 1 ) [ { 2 , :a } , { 3 , :b } ]","ref":"Bunch.KVList.html#map_keys/2","title":"Bunch.KVList.map_keys/2","type":"function"},{"doc":"Maps values of list using function f . Example iex&gt; Bunch.KVList . map_values ( [ a : 1 , b : 2 ] , &amp; &amp;1 + 1 ) [ a : 2 , b : 3 ]","ref":"Bunch.KVList.html#map_values/2","title":"Bunch.KVList.map_values/2","type":"function"},{"doc":"","ref":"Bunch.KVList.html#t:t/2","title":"Bunch.KVList.t/2","type":"type"},{"doc":"A bunch of helper functions for list manipulation.","ref":"Bunch.List.html","title":"Bunch.List","type":"module"},{"doc":"The version of unfoldr/2 that res_accepts :ok and :error return tuples. Behaves as unfoldr/2 as long as :ok tuples are returned. Upon :error the processing is stopped and error is returned. Examples iex&gt; f = fn iex&gt; &lt;&lt; a , b , rest :: binary &gt;&gt; -&gt; iex&gt; sum = a + b iex&gt; if rem ( sum , 2 ) == 1 , do : { :ok , { :cont , sum , rest } } , else : { :error , :even_sum } iex&gt; acc -&gt; { :ok , { :halt , acc } } iex&gt; end iex&gt; Bunch.List . try_unfoldr ( &lt;&lt; 1 , 2 , 3 , 4 , 5 &gt;&gt; , f ) { :ok , { [ 3 , 7 ] , &lt;&lt; 5 &gt;&gt; } } iex&gt; Bunch.List . try_unfoldr ( &lt;&lt; 2 , 4 , 6 , 8 &gt;&gt; , f ) { :error , :even_sum }","ref":"Bunch.List.html#try_unfoldr/2","title":"Bunch.List.try_unfoldr/2","type":"function"},{"doc":"Generates a list using generator function and provided initial accumulator. Successive list elements are generated by calling f with the previous accumulator. The enumeration finishes when it returns :halt or {:halt, accumulator} . Examples iex&gt; Bunch.List . unfoldr ( 10 , fn 0 -&gt; :halt ; n -&gt; { :cont , n - 1 } end ) Enum . to_list ( 9 .. 0 ) iex&gt; f = fn ...&gt; &lt;&lt; size , content :: binary - size ( size ) , rest :: binary &gt;&gt; -&gt; { :cont , content , rest } ...&gt; binary -&gt; { :halt , binary } ...&gt; end iex&gt; Bunch.List . unfoldr ( &lt;&lt; 2 , &quot;ab&quot; , 3 , &quot;cde&quot; , 4 , &quot;fghi&quot; &gt;&gt; , f ) { ~w(ab cde fghi) , &lt;&lt; &gt;&gt; } iex&gt; Bunch.List . unfoldr ( &lt;&lt; 2 , &quot;ab&quot; , 3 , &quot;cde&quot; , 4 , &quot;fg&quot; &gt;&gt; , f ) { ~w(ab cde) , &lt;&lt; 4 , &quot;fg&quot; &gt;&gt; }","ref":"Bunch.List.html#unfoldr/2","title":"Bunch.List.unfoldr/2","type":"function"},{"doc":"A bunch of helpers for implementing macros.","ref":"Bunch.Macro.html","title":"Bunch.Macro","type":"module"},{"doc":"Receives an AST and traverses it expanding all the nodes. This function uses Macro.expand/2 under the hood. Check it out for more information and examples.","ref":"Bunch.Macro.html#expand_deep/2","title":"Bunch.Macro.expand_deep/2","type":"function"},{"doc":"Imitates import functionality by finding and replacing bare function calls (like foo() ) in AST with fully-qualified call (like Some.Module.foo() ) Receives AST fragment as first parameter and a pair {Some.Module, :foo} as second","ref":"Bunch.Macro.html#inject_call/2","title":"Bunch.Macro.inject_call/2","type":"function"},{"doc":"Imitates import functionality by finding and replacing bare function calls (like foo() ) in AST with fully-qualified call (like Some.Module.foo() ) Receives AST fragment as first parameter and list of pairs {Some.Module, :foo} as second","ref":"Bunch.Macro.html#inject_calls/2","title":"Bunch.Macro.inject_calls/2","type":"function"},{"doc":"Works like Macro.prewalk/2 , but allows to skip particular nodes. Example iex&gt; code = quote do fun ( 1 , 2 , opts : [ key : :val ] ) end iex&gt; code |&gt; Bunch.Macro . prewalk_while ( fn node -&gt; ...&gt; if Keyword . keyword? ( node ) do ...&gt; { :skip , node ++ [ default : 1 ] } ...&gt; else ...&gt; { :enter , node } ...&gt; end ...&gt; end ) quote do fun ( 1 , 2 , opts : [ key : :val ] , default : 1 ) end","ref":"Bunch.Macro.html#prewalk_while/2","title":"Bunch.Macro.prewalk_while/2","type":"function"},{"doc":"Works like Macro.prewalk/3 , but allows to skip particular nodes using an accumulator. Example iex&gt; code = quote do fun ( 1 , 2 , opts : [ key : :val ] ) end iex&gt; code |&gt; Bunch.Macro . prewalk_while ( 0 , fn node , acc -&gt; ...&gt; if Keyword . keyword? ( node ) do ...&gt; { :skip , node ++ [ default : 1 ] , acc + 1 } ...&gt; else ...&gt; { :enter , node , acc } ...&gt; end ...&gt; end ) { quote do fun ( 1 , 2 , opts : [ key : :val ] , default : 1 ) end , 1 }","ref":"Bunch.Macro.html#prewalk_while/3","title":"Bunch.Macro.prewalk_while/3","type":"function"},{"doc":"A bunch of helper functions for manipulating maps.","ref":"Bunch.Map.html","title":"Bunch.Map","type":"module"},{"doc":"Works like get_updated/3 , but requires map to contain key . Uses Map.get_and_update!/3 under the hood. Example iex&gt; %{ a : 1 } |&gt; Bunch.Map . get_updated! ( :a , &amp; &amp;1 + 1 ) { 2 , %{ a : 2 } }","ref":"Bunch.Map.html#get_updated!/3","title":"Bunch.Map.get_updated!/3","type":"function"},{"doc":"Updates value at key in map and returns new value and updated map. Uses Map.get_and_update/3 under the hood. Example iex&gt; %{ a : 1 } |&gt; Bunch.Map . get_updated ( :a , &amp; &amp;1 + 1 ) { 2 , %{ a : 2 } }","ref":"Bunch.Map.html#get_updated/3","title":"Bunch.Map.get_updated/3","type":"function"},{"doc":"Maps keys of map using function f . Example iex&gt; Bunch.Map . map_keys ( %{ 1 =&gt; :a , 2 =&gt; :b } , &amp; &amp;1 + 1 ) %{ 2 =&gt; :a , 3 =&gt; :b }","ref":"Bunch.Map.html#map_keys/2","title":"Bunch.Map.map_keys/2","type":"function"},{"doc":"Maps values of map using function f . Example iex&gt; Bunch.Map . map_values ( %{ a : 1 , b : 2 } , &amp; &amp;1 + 1 ) %{ a : 2 , b : 3 }","ref":"Bunch.Map.html#map_values/2","title":"Bunch.Map.map_values/2","type":"function"},{"doc":"Works like move/3 , but fails if either old_key is absent or new_key is present in map . Example iex&gt; Bunch.Map . move! ( %{ a : 1 , b : 2 } , :a , :c ) %{ b : 2 , c : 1 }","ref":"Bunch.Map.html#move!/3","title":"Bunch.Map.move!/3","type":"function"},{"doc":"Moves value stored at old_key to new_key . If old_key is not present in map , default_value is stored at new_key . If new_key is present in map , it's value is overwritten. Examples iex&gt; Bunch.Map . move ( %{ a : 1 , b : 2 } , :a , :c , 3 ) %{ b : 2 , c : 1 } iex&gt; Bunch.Map . move ( %{ a : 1 , b : 2 } , :a , :b , 3 ) %{ b : 1 } iex&gt; Bunch.Map . move ( %{ a : 1 , b : 2 } , :c , :b , 3 ) %{ a : 1 , b : 3 }","ref":"Bunch.Map.html#move/4","title":"Bunch.Map.move/4","type":"function"},{"doc":"A bunch of helpers for generating Markdown text","ref":"Bunch.Markdown.html","title":"Bunch.Markdown","type":"module"},{"doc":"Indents the whole block of text by specified number of hard spaces ( &amp;nbsp; ). Examples iex&gt; Bunch.Markdown . hard_indent ( &quot;text&quot; ) &quot;&amp;nbsp;&amp;nbsp;text&quot; iex&gt; text = &quot;&quot;&quot; ...&gt; First line ...&gt; Second line ...&gt; Third line ...&gt; &quot;&quot;&quot; iex&gt; Bunch.Markdown . hard_indent ( text ) &quot;&quot;&quot; &amp;nbsp;&amp;nbsp;First line &amp;nbsp;&amp;nbsp;Second line &amp;nbsp;&amp;nbsp;Third line &quot;&quot;&quot; iex&gt; Bunch.Markdown . hard_indent ( text , 1 ) &quot;&quot;&quot; &amp;nbsp;First line &amp;nbsp;Second line &amp;nbsp;Third line &quot;&quot;&quot;","ref":"Bunch.Markdown.html#hard_indent/2","title":"Bunch.Markdown.hard_indent/2","type":"function"},{"doc":"Indents whole block of text by specified number of spaces Examples iex&gt; Bunch.Markdown . indent ( &quot;text&quot; ) &quot; text&quot; iex&gt; text = &quot;&quot;&quot; ...&gt; First line ...&gt; Second line ...&gt; Third line ...&gt; &quot;&quot;&quot; iex&gt; Bunch.Markdown . indent ( text ) &quot;&quot;&quot; First line Second line Third line &quot;&quot;&quot; iex&gt; Bunch.Markdown . indent ( text , 4 ) &quot;&quot;&quot; First line Second line Third line &quot;&quot;&quot;","ref":"Bunch.Markdown.html#indent/2","title":"Bunch.Markdown.indent/2","type":"function"},{"doc":"A bunch of math helper functions.","ref":"Bunch.Math.html","title":"Bunch.Math","type":"module"},{"doc":"Applies div/2 and rem/2 to arguments and returns results as a tuple. Example iex&gt; Bunch.Math . div_rem ( 10 , 4 ) { div ( 10 , 4 ) , rem ( 10 , 4 ) }","ref":"Bunch.Math.html#div_rem/2","title":"Bunch.Math.div_rem/2","type":"function"},{"doc":"Works like div_rem/2 but allows to accumulate remainder. Useful when an accumulation of division error is not acceptable, for example when you need to produce chunks of data every second but need to make sure there are 9 chunks per 4 seconds on average. You can calculate div_rem(9, 4) , keep the remainder, pass it to subsequent calls and every fourth result will be bigger than others. Example iex&gt; 1 .. 10 |&gt; Enum . map_reduce ( 0 , fn _ , err -&gt; ...&gt; Bunch.Math . div_rem ( 9 , 4 , err ) ...&gt; end ) { [ 2 , 2 , 2 , 3 , 2 , 2 , 2 , 3 , 2 , 2 ] , 2 }","ref":"Bunch.Math.html#div_rem/3","title":"Bunch.Math.div_rem/3","type":"function"},{"doc":"Returns the biggest multiple of value that is lower than or equal to threshold . Examples iex&gt; Bunch.Math . max_multiple_lte ( 4 , 10 ) 8 iex&gt; Bunch.Math . max_multiple_lte ( 2 , 6 ) 6","ref":"Bunch.Math.html#max_multiple_lte/2","title":"Bunch.Math.max_multiple_lte/2","type":"function"},{"doc":"Returns the smallest multiple of value that is greater than or equal to threshold . Examples iex&gt; Bunch.Math . min_multiple_gte ( 4 , 10 ) 12 iex&gt; Bunch.Math . min_multiple_gte ( 2 , 6 ) 6","ref":"Bunch.Math.html#min_multiple_gte/2","title":"Bunch.Math.min_multiple_gte/2","type":"function"},{"doc":"A bunch of functions for easier manipulation on modules.","ref":"Bunch.Module.html","title":"Bunch.Module","type":"module"},{"doc":"Works like Kernel.apply/3 if module exports fun_name/length(args) , otherwise returns default . Determines if function is exported using loaded_and_function_exported?/3 .","ref":"Bunch.Module.html#apply/4","title":"Bunch.Module.apply/4","type":"function"},{"doc":"Determines whether module implements a behaviour by checking a test function. Checked behaviour needs to define a callback with unique name and no arguments, that should return true . This functions ensures that the module is loaded and checks if it exports implementation of the callback that returns true . If all these conditions are met, true is returned. Otherwise returns false .","ref":"Bunch.Module.html#check_behaviour/2","title":"Bunch.Module.check_behaviour/2","type":"function"},{"doc":"Ensures that module is loaded and checks whether it exports given function.","ref":"Bunch.Module.html#loaded_and_function_exported?/3","title":"Bunch.Module.loaded_and_function_exported?/3","type":"function"},{"doc":"Returns instance of struct defined in given module, if the module defines struct. Otherwise returns nil . Raises if struct has any required fields.","ref":"Bunch.Module.html#struct/1","title":"Bunch.Module.struct/1","type":"function"},{"doc":"A bunch of helpers for handling scenarios when some actions should be repeated until it succeeds.","ref":"Bunch.Retry.html","title":"Bunch.Retry","type":"module"},{"doc":"Calls fun function until arbiter function decides to stop. Possible options are: times - limits amount of retries (first evaluation is not considered a retry) duration - limits total time of execution of this function, but breaks only before subsequent retry delay - introduces delay ( :timer.sleep/1 ) before each retry Examples iex&gt; { :ok , pid } = Agent . start_link ( fn -&gt; 0 end ) iex&gt; Bunch.Retry . retry ( fn -&gt; Agent . get_and_update ( pid , &amp; { &amp;1 , &amp;1 + 1 } ) end , &amp; &amp;1 &gt; 3 ) 4 iex&gt; { :ok , pid } = Agent . start_link ( fn -&gt; 0 end ) iex&gt; Bunch.Retry . retry ( fn -&gt; Agent . get_and_update ( pid , &amp; { &amp;1 , &amp;1 + 1 } ) end , &amp; &amp;1 &gt; 3 , times : 10 ) 4 iex&gt; { :ok , pid } = Agent . start_link ( fn -&gt; 0 end ) iex&gt; Bunch.Retry . retry ( fn -&gt; Agent . get_and_update ( pid , &amp; { &amp;1 , &amp;1 + 1 } ) end , &amp; &amp;1 &gt; 3 , times : 2 ) 2 iex&gt; { :ok , pid } = Agent . start_link ( fn -&gt; 0 end ) iex&gt; Bunch.Retry . retry ( ...&gt; fn -&gt; :timer . sleep ( 100 ) ; Agent . get_and_update ( pid , &amp; { &amp;1 , &amp;1 + 1 } ) end , ...&gt; &amp; &amp;1 &gt; 3 , ...&gt; duration : 150 ...&gt; ) 1 iex&gt; { :ok , pid } = Agent . start_link ( fn -&gt; 0 end ) iex&gt; Bunch.Retry . retry ( ...&gt; fn -&gt; :timer . sleep ( 30 ) ; Agent . get_and_update ( pid , &amp; { &amp;1 , &amp;1 + 1 } ) end , ...&gt; &amp; &amp;1 &gt; 3 , ...&gt; duration : 80 , delay : 20 ...&gt; ) 1","ref":"Bunch.Retry.html#retry/3","title":"Bunch.Retry.retry/3","type":"function"},{"doc":"Possible options for retry/3 .","ref":"Bunch.Retry.html#t:retry_option_t/0","title":"Bunch.Retry.retry_option_t/0","type":"type"},{"doc":"A wrapper over Erlang/Elixir references that makes them more readable and visually distinguishable. Erlang references When printed, Erlang references are quite long: #Reference&lt;0.133031758.722993155.68472&gt; . Moreover, since they're based on an incremented counter, it's hard to distinguish between a two created within the same period of time, like #Reference&lt;0.133031758.722993155.68512&gt; and #Reference&lt;0.133031758.722993155.68519&gt; . Bunch.ShortRef Bunch.ShortRef.t/0 stores a usual reference along with first 4 bytes of its SHA1 hash and when inspected prints only 8 hex digits prepended with # . Thanks to use of the hash function, similar references have totally different string representations - the case from the previous example would be #60e0fd2d and #d4208051 . When to use Bunch.ShortRef should be used when a reference is to be printed or logged. It should NOT be used when creating lots of references as it adds a significant performance overhead.","ref":"Bunch.ShortRef.html","title":"Bunch.ShortRef","type":"module"},{"doc":"Creates a short reference. iex&gt; IEx.Helpers . ref ( 0 , 1 , 2 , 3 ) |&gt; Bunch.ShortRef . new ( ) |&gt; inspect ( ) &quot;#82c033ef&quot; iex&gt; &lt;&lt; &quot;#&quot; , hash :: binary - size ( 8 ) &gt;&gt; = Bunch.ShortRef . new ( ) |&gt; inspect ( ) iex&gt; Base . decode16 ( hash , case : :lower ) |&gt; elem ( 0 ) :ok","ref":"Bunch.ShortRef.html#new/1","title":"Bunch.ShortRef.new/1","type":"function"},{"doc":"","ref":"Bunch.ShortRef.html#t:t/0","title":"Bunch.ShortRef.t/0","type":"type"},{"doc":"A bunch of functions for easier manipulation on structs.","ref":"Bunch.Struct.html","title":"Bunch.Struct","type":"module"},{"doc":"Wraps Bunch.Access.delete_in/2 to make it work with structs that do not implement Access behaviour.","ref":"Bunch.Struct.html#delete_in/2","title":"Bunch.Struct.delete_in/2","type":"function"},{"doc":"Wraps Bunch.Access.get_and_update_in/3 to make it work with structs that do not implement Access behaviour.","ref":"Bunch.Struct.html#get_and_update_in/3","title":"Bunch.Struct.get_and_update_in/3","type":"function"},{"doc":"Wraps Bunch.Access.get_in/2 to make it work with structs that do not implement Access behaviour.","ref":"Bunch.Struct.html#get_in/2","title":"Bunch.Struct.get_in/2","type":"function"},{"doc":"Wraps Bunch.Access.pop_in/2 to make it work with structs that do not implement Access behaviour.","ref":"Bunch.Struct.html#pop_in/2","title":"Bunch.Struct.pop_in/2","type":"function"},{"doc":"Wraps Bunch.Access.put_in/3 to make it work with structs that do not implement Access behaviour.","ref":"Bunch.Struct.html#put_in/3","title":"Bunch.Struct.put_in/3","type":"function"},{"doc":"Wraps Bunch.Access.update_in/3 to make it work with structs that do not implement Access behaviour.","ref":"Bunch.Struct.html#update_in/3","title":"Bunch.Struct.update_in/3","type":"function"},{"doc":"A bunch of commonly used types.","ref":"Bunch.Type.html","title":"Bunch.Type","type":"module"},{"doc":"Represents a value along with state.","ref":"Bunch.Type.html#t:stateful_t/2","title":"Bunch.Type.stateful_t/2","type":"type"},{"doc":"Represents a try_t/0 value along with state.","ref":"Bunch.Type.html#t:stateful_try_t/1","title":"Bunch.Type.stateful_try_t/1","type":"type"},{"doc":"Represents a try_t/1 value along with state.","ref":"Bunch.Type.html#t:stateful_try_t/2","title":"Bunch.Type.stateful_try_t/2","type":"type"},{"doc":"Represents result of an operation that may succeed or fail.","ref":"Bunch.Type.html#t:try_t/0","title":"Bunch.Type.try_t/0","type":"type"},{"doc":"Represents result of an operation that may return something or fail.","ref":"Bunch.Type.html#t:try_t/1","title":"Bunch.Type.try_t/1","type":"type"},{"doc":"A bunch of typespec-related helpers.","ref":"Bunch.Typespec.html","title":"Bunch.Typespec","type":"module"},{"doc":"Allows to define a type in form of t :: x | y | z | ... and a module parameter in form of @t [x, y, z, ...] at once. Example iex&gt; defmodule Abc do ...&gt; use Bunch.Typespec ...&gt; @list_type t :: [ :a , :b , :c ] ...&gt; @spec get_at ( 0 .. 2 ) :: t ...&gt; def get_at ( x ) , do : @t |&gt; Enum . at ( x ) ...&gt; end iex&gt; Abc . get_at ( 1 ) :b","ref":"Bunch.Typespec.html#@/1","title":"Bunch.Typespec.@/1","type":"macro"},{"doc":"Bunch is a bunch of helper functions, intended to make life easier. API documentation is available at HexDocs","ref":"readme.html","title":"Bunch","type":"extras"},{"doc":"Add the following line to your deps in mix.exs . Run mix deps.get . { :bunch , &quot;~&gt; 1.3&quot; }","ref":"readme.html#installation","title":"Bunch - Installation","type":"extras"},{"doc":"Copyright 2018, Software Mansion Licensed under the Apache License, Version 2.0","ref":"readme.html#copyright-and-license","title":"Bunch - Copyright and License","type":"extras"},{"doc":"Apache License Version 2.0, January 2004 http://www.apache.org/licenses/ TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION 1. Definitions. &quot;License&quot; shall mean the terms and conditions for use, reproduction, and distribution as defined by Sections 1 through 9 of this document. &quot;Licensor&quot; shall mean the copyright owner or entity authorized by the copyright owner that is granting the License. &quot;Legal Entity&quot; shall mean the union of the acting entity and all other entities that control, are controlled by, or are under common control with that entity. For the purposes of this definition, &quot;control&quot; means (i) the power, direct or indirect, to cause the direction or management of such entity, whether by contract or otherwise, or (ii) ownership of fifty percent (50%) or more of the outstanding shares, or (iii) beneficial ownership of such entity. &quot;You&quot; (or &quot;Your&quot;) shall mean an individual or Legal Entity exercising permissions granted by this License. &quot;Source&quot; form shall mean the preferred form for making modifications, including but not limited to software source code, documentation source, and configuration files. &quot;Object&quot; form shall mean any form resulting from mechanical transformation or translation of a Source form, including but not limited to compiled object code, generated documentation, and conversions to other media types. &quot;Work&quot; shall mean the work of authorship, whether in Source or Object form, made available under the License, as indicated by a copyright notice that is included in or attached to the work (an example is provided in the Appendix below). &quot;Derivative Works&quot; shall mean any work, whether in Source or Object form, that is based on (or derived from) the Work and for which the editorial revisions, annotations, elaborations, or other modifications represent, as a whole, an original work of authorship. For the purposes of this License, Derivative Works shall not include works that remain separable from, or merely link (or bind by name) to the interfaces of, the Work and Derivative Works thereof. &quot;Contribution&quot; shall mean any work of authorship, including the original version of the Work and any modifications or additions to that Work or Derivative Works thereof, that is intentionally submitted to Licensor for inclusion in the Work by the copyright owner or by an individual or Legal Entity authorized to submit on behalf of the copyright owner. For the purposes of this definition, &quot;submitted&quot; means any form of electronic, verbal, or written communication sent to the Licensor or its representatives, including but not limited to communication on electronic mailing lists, source code control systems, and issue tracking systems that are managed by, or on behalf of, the Licensor for the purpose of discussing and improving the Work, but excluding communication that is conspicuously marked or otherwise designated in writing by the copyright owner as &quot;Not a Contribution.&quot; &quot;Contributor&quot; shall mean Licensor and any individual or Legal Entity on behalf of whom a Contribution has been received by Licensor and subsequently incorporated within the Work. 2. Grant of Copyright License. Subject to the terms and conditions of this License, each Contributor hereby grants to You a perpetual, worldwide, non-exclusive, no-charge, royalty-free, irrevocable copyright license to reproduce, prepare Derivative Works of, publicly display, publicly perform, sublicense, and distribute the Work and such Derivative Works in Source or Object form. 3. Grant of Patent License. Subject to the terms and conditions of this License, each Contributor hereby grants to You a perpetual, worldwide, non-exclusive, no-charge, royalty-free, irrevocable (except as stated in this section) patent license to make, have made, use, offer to sell, sell, import, and otherwise transfer the Work, where such license applies only to those patent claims licensable by such Contributor that are necessarily infringed by their Contribution(s) alone or by combination of their Contribution(s) with the Work to which such Contribution(s) was submitted. If You institute patent litigation against any entity (including a cross-claim or counterclaim in a lawsuit) alleging that the Work or a Contribution incorporated within the Work constitutes direct or contributory patent infringement, then any patent licenses granted to You under this License for that Work shall terminate as of the date such litigation is filed. 4. Redistribution. You may reproduce and distribute copies of the Work or Derivative Works thereof in any medium, with or without modifications, and in Source or Object form, provided that You meet the following conditions: (a) You must give any other recipients of the Work or Derivative Works a copy of this License; and (b) You must cause any modified files to carry prominent notices stating that You changed the files; and (c) You must retain, in the Source form of any Derivative Works that You distribute, all copyright, patent, trademark, and attribution notices from the Source form of the Work, excluding those notices that do not pertain to any part of the Derivative Works; and (d) If the Work includes a &quot;NOTICE&quot; text file as part of its distribution, then any Derivative Works that You distribute must include a readable copy of the attribution notices contained within such NOTICE file, excluding those notices that do not pertain to any part of the Derivative Works, in at least one of the following places: within a NOTICE text file distributed as part of the Derivative Works; within the Source form or documentation, if provided along with the Derivative Works; or, within a display generated by the Derivative Works, if and wherever such third-party notices normally appear. The contents of the NOTICE file are for informational purposes only and do not modify the License. You may add Your own attribution notices within Derivative Works that You distribute, alongside or as an addendum to the NOTICE text from the Work, provided that such additional attribution notices cannot be construed as modifying the License. You may add Your own copyright statement to Your modifications and may provide additional or different license terms and conditions for use, reproduction, or distribution of Your modifications, or for any such Derivative Works as a whole, provided Your use, reproduction, and distribution of the Work otherwise complies with the conditions stated in this License. 5. Submission of Contributions. Unless You explicitly state otherwise, any Contribution intentionally submitted for inclusion in the Work by You to the Licensor shall be under the terms and conditions of this License, without any additional terms or conditions. Notwithstanding the above, nothing herein shall supersede or modify the terms of any separate license agreement you may have executed with Licensor regarding such Contributions. 6. Trademarks. This License does not grant permission to use the trade names, trademarks, service marks, or product names of the Licensor, except as required for reasonable and customary use in describing the origin of the Work and reproducing the content of the NOTICE file. 7. Disclaimer of Warranty. Unless required by applicable law or agreed to in writing, Licensor provides the Work (and each Contributor provides its Contributions) on an &quot;AS IS&quot; BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied, including, without limitation, any warranties or conditions of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A PARTICULAR PURPOSE. You are solely responsible for determining the appropriateness of using or redistributing the Work and assume any risks associated with Your exercise of permissions under this License. 8. Limitation of Liability. In no event and under no legal theory, whether in tort (including negligence), contract, or otherwise, unless required by applicable law (such as deliberate and grossly negligent acts) or agreed to in writing, shall any Contributor be liable to You for damages, including any direct, indirect, special, incidental, or consequential damages of any character arising as a result of this License or out of the use or inability to use the Work (including but not limited to damages for loss of goodwill, work stoppage, computer failure or malfunction, or any and all other commercial damages or losses), even if such Contributor has been advised of the possibility of such damages. 9. Accepting Warranty or Additional Liability. While redistributing the Work or Derivative Works thereof, You may choose to offer, and charge a fee for, acceptance of support, warranty, indemnity, or other liability obligations and/or rights consistent with this License. However, in accepting such obligations, You may act only on Your own behalf and on Your sole responsibility, not on behalf of any other Contributor, and only if You agree to indemnify, defend, and hold each Contributor harmless for any liability incurred by, or claims asserted against, such Contributor by reason of your accepting any such warranty or additional liability. END OF TERMS AND CONDITIONS APPENDIX: How to apply the Apache License to your work. To apply the Apache License to your work, attach the following boilerplate notice, with the fields enclosed by brackets &quot;[]&quot; replaced with your own identifying information. (Don't include the brackets!) The text should be enclosed in the appropriate comment syntax for the file format. We also recommend that a file or class name and description of purpose be included on the same &quot;printed page&quot; as the copyright notice for easier identification within third-party archives. Copyright 2018 Software Mansion Licensed under the Apache License, Version 2.0 (the &quot;License&quot;); you may not use this file except in compliance with the License. You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0 Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on an &quot;AS IS&quot; BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the specific language governing permissions and limitations under the License.","ref":"license.html","title":"LICENSE","type":"extras"}]